(* Drucken von PDF-Dateien mit Foxit Reader

   2011  J. Rathlev, IEAP, Uni-Kiel, (info(a)rathlev-home.de)

 -----------------------------------------------------------------------------
   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1,   Mar. 2006 : mit Adobe Reader 
   Vers. 2,   Nov. 2009 : mit Foxit Reader
   Vers. 3,   Jan. 2012 : mit Delphi XE2
   *)

unit FPdfPrint;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, HListBox, JrUtils, WinShell, ComCtrls,
  Printers, ExtCtrls, XPMan, Spin, NumberEd, WinApiUtils;

const
  ProgName = 'Ausdruck von PDF-Dateien';
  Vers = ' (Vers. 3.0)';
  CopRgt = ' 2012 J. Rathlev (IEAP, Uni-Kiel)';
  EmailAdr = 'info(a)rathlev-home.de';

  BatExt = 'pbt';       // batch file list - Format: Filename

  defFoxitPath = 'Foxit Software\Foxit Reader\Foxit Reader.exe';

type
  THauptForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    InfoBtn: TSpeedButton;
    EndeBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    SourceDirEdit: THistoryCombo;
    FileEdit: THistoryCombo;
    SourceSelectBtn: TSpeedButton;
    FileBtn: TSpeedButton;
    chbSubDir: TCheckBox;
    btnProcess: TBitBtn;
    PrtSelectBtn: TBitBtn;
    StatusBar: TStatusBar;
    lblStatus: TLabel;
    PrinterSetupDialog: TPrinterSetupDialog;
    Label3: TLabel;
    KopAnz: TSpinEdit;
    btnEdit: TBitBtn;
    btnNew: TBitBtn;
    gbxKop: TGroupBox;
    Label4: TLabel;
    edtKop: TNumberEdit;
    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure btnProcessClick(Sender: TObject);
    procedure SourceSelectBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PrtSelectBtnClick(Sender: TObject);
    procedure FileEditChange(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    AppPath,UserPath,
    ProgPath,IniName,
    PrtName,
    FoxitPath        : string;
    Printing,Stopped : boolean;
    hAcro            : HWnd;

  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.DFM}
{$R *.RES}

uses ShellDirDlg, ShlObj, WinExecute, StrUtils, FileListDlg, IniFiles,
  FileUtils, LangUtils, GnuGetText, InitProg;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  SourceSekt = 'Source';
  DestSekt = 'Dest';
  FileSekt = 'Files';

  (* INI-Variablen *)
  iniDir = 'Direction';
  iniSubDir = 'Subdirectories';
  iniPrtName = 'Printer';
  iniFoxitPath = 'FoxitReader';

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  TranslateComponent(self);
  InitPaths(AppPath,UserPath,ProgPath);
  InitVersion(ProgName,Vers,CopRgt,2,2,ProgVersName,ProgVersDate);
  Caption:='PDF Print'+VersInfo.Comments;
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    FoxitPath:=ReadString(CfgSekt,IniFoxitPath,'');
    PrtName:=ReadString(CfgSekt,IniPrtName,'');
    chbSubDir.Checked:=ReadBool(CfgSekt,iniSubDir,false);
    Free;
    end;
  with SourceDirEdit do begin
    LoadFromIni (IniName,SourceSekt);
    ItemIndex:=0;
    end;
  with FileEdit do begin
    LoadFromIni (IniName,FileSekt);
    if Items.Count=0 then AddItem('*.pdf');
    ItemIndex:=0;
    end;
  Printing:=false; Stopped:=false;
  lblStatus.Caption:='';
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteString(CfgSekt,IniFoxitPath,FoxitPath);
    WriteString(CfgSekt,IniPrtName,PrtName);
    WriteBool(CfgSekt,iniSubDir,chbSubDir.Checked);
    Free;
    end;
  if hAcro<>0 then StopProcess(hAcro);
  end;

procedure THauptForm.FormShow(Sender: TObject);
var
  s : string;
  ok : boolean;
begin
  with Printer do if Printers.Count>0 then begin
    PrinterIndex:=Printers.IndexOf(PrtName);
    PrtName:=Printers[PrinterIndex];
    StatusBar.SimpleText:=PrtName;
    end
  else begin
    ErrorDialog(Caption,_('No printer available!'));
    Close;
    end;
  if (length(FoxitPath)=0) or not FileExists(FoxitPath) then begin
    s:=GetDesktopFolder(CSIDL_PROGRAM_FILES);
    FoxitPath:=IncludeTrailingPathDelimiter(s)+defFoxitPath;
    ok:=FileExists(FoxitPath);
    if not ok then begin
      with OpenDialog do begin
        InitialDir:=s;
        Filename:='';
        Filter:=_('Programs|*.exe'); //'Programme|*.exe'
        Title:=_('Seek for Foxit Reader');   //'Foxit Reader suchen'
        if Execute then FoxitPath:=Filename
        else begin
          //'Foxit Reader nicht gefunden!'+sLineBreak+'Kein Ausdruck von PDF-Dateien mglich!
          ErrorDialog(Caption,_('Foxit Reader not found!'+sLineBreak+'Printing of PDF files not available!'));
          Close;
          end;
        end;
      end;
    end;
  with btnProcess do begin
    Glyph.LoadFromResourceName(HInstance,'START');
    Caption:=_('Print');        //'Drucken';
    Hint:=_('Start printing');  //'Ausdruck starten'
    end;
  FileEditChange(Sender);
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog('',ProgVersName+' - '+ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr+sLineBreak+
           _('Installation of Foxit Reader required:')+sLineBreak+
           'http://www.foxitsoftware.com/pdf/reader/',
           CursorPos);
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.SourceSelectBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=SourceDirEdit.Text;
  if length(s)=0 then s:=UserPath;
  // 'Verzeichnis mit den zu druckenden Dateien'
  if ShellDirDialog.Execute (_('Directory with files to print'),true,true,'',s) then begin
    with SourceDirEdit do begin
      Text:=s; AddItem(s);
      end;
    end;
  end;

procedure THauptForm.FileBtnClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  with OpenDialog do begin
    InitialDir:=SourceDirEdit.Text;
    Filename:='';
    //'PDF-Dateien|*.pdf|Drucklisten|*.%s'
    Filter:=Format(_('PDF files|*.pdf|Print list|*.%s'),[BatExt]);
    Title:=_('Select files to print or print list'); //'Zu druckende Dateien auswhlen'
    if Execute then begin
      chbSubDir.Checked:=false;
      s:=ExtractFilePath(Files.Strings[0]);
      with SourceDirEdit do if s<>Text then begin
        Text:=s; AddItem(s);
        end;
      with Files do begin
        s:='';
        for i:=0 to Count-1 do s:=s+ExtractFilename(Strings[i])+';';
        end;
      delete (s,length(s),1);
      with FileEdit do begin
        Text:=s; AddItem(s);
        end;
      end;
    end;
  end;

procedure THauptForm.btnProcessClick(Sender: TObject);
var
  FListe        : TStringList;
  fcnt,i,j      : integer;
  bf            : TextFile;
  s,t,fname     : string;

  procedure ScanDir (Dir,Mask : string);
  var
    DirInfo    : TSearchRec;
    Findresult : integer;
  begin
    if chbSubDir.Checked then begin
      FindResult:=FindFirst (Erweiter(Dir,'*','*'),faAnyFile,DirInfo);
      while FindResult=0 do with DirInfo do begin
        if NotSpecialDir(Name) then begin
          if((Attr and faDirectory)<>0) then ScanDir (Erweiter(Dir,Name,''),Mask);
          end;
        FindResult:=FindNext (DirInfo);
        end;
      FindClose(DirInfo);
      end;
    FindResult:=FindFirst (Erweiter(Dir,Mask,''),faArchive+faReadOnly+faHidden+faSysfile,DirInfo);
    while FindResult=0 do with DirInfo do begin
      if NotSpecialDir(Name) then FListe.Add(Erweiter(Dir,Name,''));
      FindResult:=FindNext (DirInfo);
      end;
    FindClose(DirInfo);
    end;

begin
  if Printing then Stopped:=true
  else begin
    t:=FileEdit.Text;
    if (length(t)>0) then begin
      Printing:=true; 
      with lblStatus do begin
        Font.Color:=clBlack;
        Caption:='';
        end;
      FListe:=TStringList.Create;
      fcnt:=0;
      with btnProcess do begin
        Glyph.LoadFromResourceName(HInstance,'STOP');
        Caption:=_('Stop');  // 'Stopp'
        Hint:=_('Stop printing');  //'Ausdruck anhalten'
        end;
      FListe.Clear;
      repeat
        s:=ReadNxtStr (t,';');
        if LowerCase(GetExt(s))=BatExt then begin // get files from batch list
          s:=Erweiter(SourceDirEdit.Text,s,'');
          if FileExists (s) then begin
            AssignFile(bf,s); reset(bf);
            while not Eof(bf) do begin
              readln (bf,s);
              FListe.Add(Erweiter(SourceDirEdit.Text,s,''));
              end;
            CloseFile(bf);
            end
          else ErrorDialog(Caption,Format(_('Print list: %s not found!'),[s]));  //'Druckliste: %s nicht gefunden!'
          end
        else ScanDir (SourceDirEdit.Text,s);
        until (length(t)=0);
      for j:=1 to KopAnz.Value do begin
        edtKop.Value:=j;
        with FListe do if Count>0 then for i:=0 to Count-1 do begin
          if Stopped then Break;
          fName:=Strings[i];
          lblStatus.Caption:=Format(_('File: %s'),[StripPath(fName,50)]);  //'Datei: %s'
          s:=AnsiQuotedStr(FoxitPath,'"')+' /t '+AnsiQuotedStr(fName,'"')+' '+AnsiQuotedStr(PrtName,'"');
          if ExecuteProcess (s,SourceDirEdit.Text,false,0)<0 then begin
            //'Datei: %s'+CrLf+' konnte nicht gedruckt werden!'
            if MessageDialog (Caption,Format(_('File: %s'+sLineBreak+'could not be printed!'),[fName]),
                  mtError,[mbIgnore,mbCancel])=mrCancel then Stopped:=true;
            end
          else begin
            inc(fcnt);
            Sleep(1000);  // kleine Pause
            end;
          end;
        if Stopped then Break;
        end;
      FListe.Free;
      with lblStatus do begin
        Font.Color:=clBlue;
        if fcnt=0 then Caption:=_(' No file printed')        //' Es wurden keine Dateien gedruckt'
        else if fcnt=1 then Caption:=_(' 1 file printed')    //' Es wurde 1 Datei gedruckt'
        else Caption:=Format(_(' %u files printed'),[fcnt]); //' Es wurden %u Dateien gedruckt'
        end;
      with btnProcess do begin
        Glyph.LoadFromResourceName(HInstance,'START');
        Caption:=_('Print');        //'Drucken';
        Hint:=_('Start printing');  //'Ausdruck starten'
        end;
      Printing:=false; Stopped:=false;
      end
    else lblStatus.Caption:=_('Nothing to do!');  //' Nichts zu tun!!'
    end;
  end;

procedure THauptForm.PrtSelectBtnClick(Sender: TObject);
begin
  if not Printing then begin
    if PrinterSetupDialog.Execute then with Printer do begin
      PrtName:=Printers[PrinterIndex];
      StatusBar.SimpleText:=PrtName;
      end;
    end;
  end;

procedure THauptForm.FileEditChange(Sender: TObject);
begin
  chbSubDir.Enabled:=AnsiContainsText(FileEdit.Text,'*') or
                     AnsiContainsText(FileEdit.Text,'?');
  end;

procedure THauptForm.btnEditClick(Sender: TObject);
var
  s : string;
begin
  with OpenDialog do begin
    InitialDir:=SourceDirEdit.Text;
    if AnsiContainsText(FileEdit.Text,BatExt) then
      Filename:=Copy(FileEdit.Text,1,AnsiPos(BatExt,FileEdit.Text)+3)
    else Filename:='';
    Filter:=Format(_('Print list|*.%s'),[BatExt]);  //'Druckliste|*.%s'
    Title:=_('Select print list');    //'Druckliste auswhlen'
    if Execute then begin
      s:=FileListDialog.Execute(_('Edit print list'),_('PDF files|*.pdf'),   //'Druckliste bearbeiten' 'PDF-Dateien|*.pdf'
        SourceDirEdit.Text,Filename,Filter,BatExt);
      if length(s)>0 then begin
        FileEdit.Text:=ExtractFilename(s);
        s:=ExtractFilePath(s);
        with SourceDirEdit do if s<>Text then begin
          Text:=s; AddItem(s);
          end;
        end;
      end;
    end;
  end;

procedure THauptForm.btnNewClick(Sender: TObject);
var
  s : string;
begin
  s:=FileListDialog.Execute(_('Create print list'),_('PDF files|*.pdf'),   //'Druckliste erstellen'
       SourceDirEdit.Text,'',Format(_('Batch files|*.%s'),[BatExt]),BatExt);
  if length(s)>0 then begin
    FileEdit.Text:=ExtractFilename(s);
    s:=ExtractFilePath(s);
    with SourceDirEdit do if s<>Text then begin
      Text:=s; AddItem(s);
      end;
    end;
  end;

end.
