(* Convert Pdf to Text
   GUI for command line tool PdfToText.exe
   
   2011  J. Rathlev, IEAP, Uni-Kiel, (info(a)rathlev-home.de)

 -----------------------------------------------------------------------------
   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Jul. 2009
   *)

unit P2TMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, HListBox, XPMan, Menus;

const
  Prog = 'PDF to Text';
  Vers = ' (Vers. 1.0)';
  CopRgt = ' 2009 J. Rathlev (IEAP, Uni-Kiel)';
  EmailAdr = 'info(a)rathlev-home.de';

  defPdfToText = 'pdftotext.exe';

type
  TMainForm = class(TForm)
    pnTop: TPanel;
    bbLoad: TBitBtn;
    bbFileList: TBitBtn;
    pnlRight: TPanel;
    bbInfo: TBitBtn;
    mmText: TMemo;
    laFile: TLabel;
    bbSave: TBitBtn;
    bbTools: TBitBtn;
    XPManifest1: TXPManifest;
    OpenDialog: TOpenDialog;
    pmFiles: TPopupMenu;
    SaveDialog: TSaveDialog;
    bbCopy: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure bbLoadClick(Sender: TObject);
    procedure bbSaveClick(Sender: TObject);
    procedure bbToolsClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bbFileListClick(Sender: TObject);
    procedure bbInfoClick(Sender: TObject);
    procedure bbCopyClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgVersName,
    ProgVersDate,
    AppPath,UserPath,
    ProgPath,IniName,
    PdfName,PdfToText : string;
    FileList          : THistoryList;
    function InitPdfToText : boolean;
    procedure LoadPdfFile (const FileName : string);
    procedure SaveText (const FileName : string);
    procedure LoadFromFileListClick (Sender : TObject; FileName : string);
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses JrUtils, WinApiUtils, WinExecute, IniFiles, ClipBrd,
  GnuGetTextInit, GnuGetText;

{ ------------------------------------------------------------------- }
const
  IniExt = 'ini';
  CfgSekt = 'Config';
  FileSekt = 'Files';

  IniLeft= 'Left';
  IniTop = 'Top';
  IniWidth = 'Width';
  IniHeight = 'Height';

  iniLast = 'LastFile';
  iniPdfToText = 'PdfToText';

procedure TMainForm.FormCreate(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  TranslateComponent(self);
  InitPaths(AppPath,UserPath,ProgPath);
  InitVersion(Prog,Vers,CopRgt,ProgVersName,ProgVersDate);
  Caption:='PDF View'+VersInfo.Comments;
  IniName:=Erweiter(AppPath,PrgName,IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    Left:=ReadInteger (CfgSekt,IniLeft,Left);
    Top:=ReadInteger (CfgSekt,IniTop,Top);
    ClientWidth:=ReadInteger (CfgSekt,IniWidth,ClientWidth);
    ClientHeight:=ReadInteger (CfgSekt,IniHeight,ClientHeight);
    PdfName:=ReadString(CfgSekt,iniLast,'');
    PdfToText:=ReadString(CfgSekt,iniPdfToText,defPdfToText);
    if not ContainsFullPath(PdfToText) then PdfToText:=PrgPath+PdfToText;
    if not FileExists(PdfToText) then PdfToText:='';
    Free;
    end;
  Caption:=GetVersionName(Application.Exename,Prog,Vers);
  (* expand file menu *)
  FileList:=THistoryList.Create;
  with FileList do begin
    LoadFromIni (IniName,FileSekt);
    Menu:=pmFiles.Items;
    OnAutoItemClick:=LoadFromFileListClick;
    end;
  bbSave.Enabled:=false;
  end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  with TIniFile.Create(IniName) do begin
    WriteInteger(CfgSekt,IniLeft,Left);
    WriteInteger(CfgSekt,IniTop,Top);
    WriteInteger(CfgSekt,IniWidth,ClientWidth);
    WriteInteger(CfgSekt,IniHeight,ClientHeight);
    WriteString(CfgSekt,iniLast,PdfName);
    WriteString(CfgSekt,iniPdfToText,PdfToText);
    Free;
    end;
  FileList.SaveToIni (IniName,FileSekt,true);
  end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FileList.Free;
  end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  if not FileExists(PdfToText) and not InitPdfToText then Close;
  end;

{ ------------------------------------------------------------------- }
// Prozess mit Befehlszeile "CmdLine" starten
// Ausgabe von StdOut wird in Stringliste bertragen
function ExecuteProcess (CmdLine,WorkDir : string; AText : TMemo) : boolean;
const
  BUFSIZE = 1024;
var
  si        : TStartupInfo;
  pi        : TProcessInformation;
  s         : string;
  saAttr    : TSecurityAttributes;
  hChildStdoutRd,hChildStdoutWr,
  hChildStdoutRdDup  : THandle;
  chBuf     : array [0..BUFSIZE] of char;
  dwread    : DWord;
  n         : integer;
begin
  Result:=false;
  AText.Clear;
// Set the bInheritHandle flag so pipe handles are inherited.
  saAttr.nLength:=sizeof(SECURITY_ATTRIBUTES);
  saAttr.bInheritHandle:=TRUE;
  saAttr.lpSecurityDescriptor:=nil;

 // The steps for redirecting child process's STDOUT:
 //     1. Save current STDOUT, to be restored later.
 //     2. Create anonymous pipe to be STDOUT for child process.
 //     3. Set STDOUT of the parent process to be write handle to
 //        the pipe, so it is inherited by the child process.
 //        (this is made instead by CreateProcess - StartupInfo, JR)
 //     4. Create a noninheritable duplicate of the read handle and
 //        close the inheritable read handle.

// Save the handle to the current STDOUT.
//    hSaveStdout:=GetStdHandle(STD_OUTPUT_HANDLE);
// Create a pipe for the child process's STDOUT.
  if not CreatePipe(hChildStdoutRd,hChildStdoutWr,@saAttr, 0) then exit;
// Set a write handle to the pipe to be STDOUT.
{    if not SetStdHandle(STD_OUTPUT_HANDLE,hChildStdoutWr) then begin
    ShowError('Redirecting STDOUT failed'); exit;
    end;
    }
// Create noninheritable read handle and close the inheritable read
// handle.
  if not DuplicateHandle(GetCurrentProcess,hChildStdoutRd,GetCurrentProcess,
                         @hChildStdoutRdDup , 0, FALSE,
                         DUPLICATE_SAME_ACCESS) then exit;
  CloseHandle(hChildStdoutRd);

// Create process to start Program
  FillChar(si, SizeOf(TStartupInfo), 0);
  with si do begin
    cb:=Sizeof(TStartupInfo);
    dwFlags:=STARTF_USESHOWWINDOW;
    dwFlags:=dwFlags or STARTF_USESTDHANDLES;
    wShowWindow:=SW_HIDE;
    hStdOutput :=hChildStdoutWr;
    end;

  if CreateProcess(nil,                // Anwendungsname
                   pchar(CmdLine),
                   nil,                // Security
                   nil,                // Security
                   true,               // use InheritHandles
                   NORMAL_PRIORITY_CLASS, // Prioritt
                   nil,                   // Environment
                   pchar(WorkDir),     // Verzeichnis
                   si,pi) then begin

    // wait for end or timeout
    n:=10;  // time in 1/10 s
    repeat
      Result:=WaitForSingleObject(pi.hProcess,100)=WAIT_OBJECT_0;
      Application.ProcessMessages;
      dec(n);
      until Result or (n=0);   // funktioniert nur beim ersten Mal
    if n=0 then begin
      TerminateProcess(pi.hProcess,0);  // Prozess beenden bei Timeout
      Result:=true;
      end;
    CloseHandle(pi.hProcess);

// After process creation, restore the saved STDOUT.
// Close the write end of the pipe before reading from the
// read end of the pipe.
    if not CloseHandle(hChildStdoutWr) then begin
      Result:=false; exit;
      end;
    if Result then begin
      chBuf[BUFSIZE]:=#0;
  // Read output from the child process, and write to parent's STDOUT.
      while ReadFile(hChildStdoutRdDup,chBuf[0],BUFSIZE,dwRead,nil)
            and (dwRead=BUFSIZE) do begin
        s:=chBuf;
        with AText do Text:=Text+s;
        end;
      if dwRead>0 then begin
        chBuf[dwread]:=#0;
        s:=chBuf;
        with AText do Text:=Text+s;
        end;
      CloseHandle(hChildStdoutRdDup);
      end;
    end
  end;

{ ------------------------------------------------------------------- }
function TMainForm.InitPdfToText : boolean;
begin
  with OpenDialog do begin
    if length(PdfToText)>0 then InitialDir:=ExtractFilePath(PdfToText)
    else InitialDir:=ProgPath;
    Filename:=defPdfToText;
    Filter:=_('Programs|*.exe');
    Title:=_('Search for PdfToText.exe');
    if Execute then begin
      PdfToText:=Filename;
      Result:=true;
      end
    else Result:=false;
    end;
  end;

// Load Pdf file
procedure TMainForm.LoadPdfFile (const FileName : string);
var
  s : string;
begin
  Cursor:=crHourGlass;
  Application.ProcessMessages;
  if FileExists(PdfToText) then begin
    s:='"'+PdfToText+'" -f 1  -nopgbrk -enc Latin1 "'+FileName+'" -';
    if not ExecuteProcess(s,ExtractFilePath(FileName),mmText) then
      ErrorDialog('',_('Execution of PdfToText.exe failed!'));
    if mmText.Lines.Count>0 then begin
      bbSave.Enabled:=true;
      PdfName:=Filename;
      laFile.Caption:=Filename;
      FileList.AddString(FileName);
      end;
    end
  else ErrorDialog('',_('PdfToText.exe not found!'));
  Cursor:=crDefault;
  end;

procedure TMainForm.SaveText (const FileName : string);
begin
  mmText.Lines.SaveToFile(FileName);
  end;

(* click on recent files *)
procedure TMainForm.LoadFromFileListClick (Sender     : TObject;
                                           FileName   : string);
begin
  if FileExists(FileName) then LoadPdfFile(FileName)
  else begin
    ErrorDialog(_('File not found!'));
    FileList.RemString(FileName);
    end;
  end;

procedure TMainForm.bbLoadClick(Sender: TObject);
begin
  with OpenDialog do begin
    if length(PdfName)>0 then InitialDir:=ExtractFilePath(PdfName)
    else InitialDir:=UserPath;
    Filename:='';
    Filter:=_('PDF files|*.pdf');
    Title:=_('Open Pdf document');
    if Execute then LoadPdfFile(FileName);
    end;
  end;

procedure TMainForm.bbSaveClick(Sender: TObject);
begin
  with SaveDialog do begin
    InitialDir:=ExtractFilePath(PdfName);
    if Execute then SaveText(FileName);
    end;
  end;

procedure TMainForm.bbToolsClick(Sender: TObject);
begin
  InitPdfToText;
  end;

procedure TMainForm.bbFileListClick(Sender: TObject);
begin
  with CursorPos do pmFiles.Popup(x,y);
  end;

procedure TMainForm.bbInfoClick(Sender: TObject);
begin
  InfoDialog('',ProgVersName+' - '+ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr,CursorPos);
  end;

procedure TMainForm.bbCopyClick(Sender: TObject);
begin
  with mmText do begin
    if SelLength=0 then SelectAll;
    CopyToClipBoard;
    SelLength:=0;
    end;
  end;

end.
